/*
 *	DATA.C
 *	Tom Kerrigan's Simple Chess Program (TSCP)
 *
 *	Copyright 1997 Tom Kerrigan
 *
 *  Traduccion al castellano por Jose Carlos Martinez Galan
 *	por sugerencia de Roger Davis
 *
 */


#include "defs.h"


/* Representacion del tablero */
int color[64];  /* LIGHT, DARK, or EMPTY : blanco, negro o vacio */
int piece[64];  /* PAWN, KNIGHT, BISHOP, ROOK, QUEEN, KING, or EMPTY :
                   peon, caballo, alfil, torre, dama, rey o vacio */
int side;  /* Bando que mueve */
int xside;  /* Bando que no mueve */
int castle;  /* Derechos de enroque por bit (se emplean los 4 menos significativos:
			    0001: las blancas pueden enrocar corto,
				0010: las blancas pueden enrocar largo,
				0100: las negras pueden enrocar corto,
				1000: las negras pueden enrocar largo.
				Las combinaciones de 1 y 0 determinan los derechos para ambos bandos */
int ep;  /* Casilla "al paso". Si las blancas mueven e2e4, la casilla al paso se establece
            en e3, pues esa es la unica casilla donde un peon negro podria capturar al paso */
int fifty;  /* Numero de jugadas desde la ultima captura o jugada de peon. Se emplea para
               la regla de las 50 jugadas */
int ply;  /* El medio-movimiento en el que nos encontramos */

/* gen_dat es una zona de memoria reservada para las listas de movimientos
   creadas por los generadores de movimientos. La lista de jugadas para el
   ply n comienza en gen_begin[n] de gen_dat. gen_end[n] esta justo detras
   de la lista de movimientos */
gen_t gen_dat[MOVE_STACK];
int gen_begin[HIST_STACK], gen_end[HIST_STACK];

/* El array de la heuristica de historia (usada para la ordenacion
   de movimientos */
int history[64][64];

/* Necesitamos un array de hist_t's para poder retroceder los movimientos */
hist_t hist_dat[HIST_STACK];

long nodes;  /* Numero de nodos buscados (visitados) */

/* El array triangular para gestionar la PV (variante principal) */
move pv[HIST_STACK][HIST_STACK];
int pv_length[HIST_STACK];
BOOL follow_pv;

/* undo_dat almacena los movimientos de la partida (no del analisis)
   para que el usuario pueda retrocederlos */
hist_t undo_dat[UNDO_STACK];
int undos;


/* Ahora tenemos el array mailbox, asi llamado porque se asemeja a un
   apartado de correos, al menos segun Bob Hyatt. Es util para conocer
   adonde puede ir cada pieza. Supongamos que tenemos una torre en a4 (32)
   y queremos saber si se pude mover una casilla hacia la izquierda.
   Restamos 1 y tenemos 32 (h5). La torre no puede moverse a h5, pero
   no podemos saber eso sin haver bastante trabajo. Asi que lo que hacemos
   es buscar el apartado de correos de la casilla a4, que es 61. Le
   restamos 1 y tenemos 60, y comprobamos el apartado de correos de 60
   (mailbox[60]). En este caso es -1, asi que esta fuera del tablero y
   podemos olvidarlo. mailbox[] se usa en attack() en board.c */

int mailbox[120] = {
	 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
	 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
	 -1,  0,  1,  2,  3,  4,  5,  6,  7, -1,
	 -1,  8,  9, 10, 11, 12, 13, 14, 15, -1,
	 -1, 16, 17, 18, 19, 20, 21, 22, 23, -1,
	 -1, 24, 25, 26, 27, 28, 29, 30, 31, -1,
	 -1, 32, 33, 34, 35, 36, 37, 38, 39, -1,
	 -1, 40, 41, 42, 43, 44, 45, 46, 47, -1,
	 -1, 48, 49, 50, 51, 52, 53, 54, 55, -1,
	 -1, 56, 57, 58, 59, 60, 61, 62, 63, -1,
	 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
	 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1
};

int mailbox64[64] = {
	21, 22, 23, 24, 25, 26, 27, 28,
	31, 32, 33, 34, 35, 36, 37, 38,
	41, 42, 43, 44, 45, 46, 47, 48,
	51, 52, 53, 54, 55, 56, 57, 58,
	61, 62, 63, 64, 65, 66, 67, 68,
	71, 72, 73, 74, 75, 76, 77, 78,
	81, 82, 83, 84, 85, 86, 87, 88,
	91, 92, 93, 94, 95, 96, 97, 98
};


/* slide, offsets, and offset son, basicamente, los vectores
   por los que las piezas se pueden mover. Si slide es FALSE
   para una pieza, esta solo se puede mover una casilla en
   cada direccion. offsets es el numero de direcciones a las
   que se puede mover y offset es un array con las direcciones
   reales */

BOOL slide[6] = {
	FALSE, FALSE, TRUE, TRUE, TRUE, FALSE
};

int offsets[6] = {
	0, 8, 4, 4, 8, 8
};

int offset[6][8] = {
	{ 0, 0, 0, 0, 0, 0, 0, 0 },
	{ -21, -19, -12, -8, 8, 12, 19, 21 },
	{ -11, -9, 9, 11, 0, 0, 0, 0 },
	{ -10, -1, 1, 10, 0, 0, 0, 0 },
	{ -11, -10, -9, -1, 1, 9, 10, 11 },
	{ -11, -10, -9, -1, 1, 9, 10, 11 }
};


/* Este es el array de mascara de enroque. Lo podemos usar para
   determinar los derechos de enroque tras un movimiento. Lo que
   hacemos es el AND-logico de los bits de enroque con los bits
   de la mascara para ambas casillas del movimiento. Digamos que
   castle es 1 (las blancas pueden enrocar corto). Ahora hacemos
   un movimiento en el que se captura la torre de h1. Hacemos el
   AND de castle con castle_mask[63], asi que tenemos 1&14, y
   castle se queda en 0, con lo que el blanco ya no puede enrocar
   corto */

int castle_mask[64] = {
	 7, 15, 15, 15,  3, 15, 15, 11,
	15, 15, 15, 15, 15, 15, 15, 15,
	15, 15, 15, 15, 15, 15, 15, 15,
	15, 15, 15, 15, 15, 15, 15, 15,
	15, 15, 15, 15, 15, 15, 15, 15,
	15, 15, 15, 15, 15, 15, 15, 15,
	15, 15, 15, 15, 15, 15, 15, 15,
	13, 15, 15, 15, 12, 15, 15, 14
};


/* Las letras de las piezas, para imprimir el tablero */
char piece_char[6] = {
	'P', 'N', 'B', 'R', 'Q', 'K'
};


/* La posicion inicial del tablero */

int init_color[64] = {
	1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1,
	6, 6, 6, 6, 6, 6, 6, 6,
	6, 6, 6, 6, 6, 6, 6, 6,
	6, 6, 6, 6, 6, 6, 6, 6,
	6, 6, 6, 6, 6, 6, 6, 6,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0
};

int init_piece[64] = {
	3, 1, 2, 4, 5, 2, 1, 3,
	0, 0, 0, 0, 0, 0, 0, 0,
	6, 6, 6, 6, 6, 6, 6, 6,
	6, 6, 6, 6, 6, 6, 6, 6,
	6, 6, 6, 6, 6, 6, 6, 6,
	6, 6, 6, 6, 6, 6, 6, 6,
	0, 0, 0, 0, 0, 0, 0, 0,
	3, 1, 2, 4, 5, 2, 1, 3
};
