/*
 *	SEARCH.C
 *	Tom Kerrigan's Simple Chess Program (TSCP)
 *
 *	Copyright 1997 Tom Kerrigan
 *
 *  Traduccion al castellano por Jose Carlos Martinez Galan
 *	por sugerencia de Roger Davis
 *
 */


#include <stdio.h>
#include <string.h>
#include "defs.h"
#include "data.h"
#include "protos.h"


/* think() llama a search() iterativamente. Si quiet es FALSE,
   imprime el analisis tras cada iteracion */

void think(int skill, BOOL quiet)
{
	int i, j, x;

	ply = 0;
	nodes = 0;
	memset(pv, 0, sizeof(pv));
	memset(history, 0, sizeof(history));
	if (!quiet)
		printf("ply      nodes  score  pv\n");
	for (i = 1; i <= skill; ++i) {
		follow_pv = TRUE;
		x = search(-10000, 10000, i);
		if (!quiet) {
			printf("%3d  %9ld  %5d ", i, nodes, x);
			for (j = 0; j < pv_length[0]; ++j) {
				printf(" ");
				print_move(pv[0][j].b);
			}
			printf("\n");
		}
	}
}


/* search() realiza una busqueda al estilo negamax */
// La busqueda en TSCP sigue el algoritmo conocido como
// Alfa-Beta. Este algoritmo se basa en dos limites
// (bounds) que nos indican en que intervalo se encuentran
// los valores que buscamos. Alfa representa el mejor
// valor hallado hasta el momento a nuestro favor, mientras
// que beta es un valor "tan bueno" para nosotros, que si
// logramos superarlo, nuestro adversario elegira otro
// movimiento mas arriba en el arbol. A esto se le llama
// Beta-cutoff, es decir, cuando logramos un valor tan alto
// que no vale la pena seguir analizando el resto de las
// jugadas, pues con seguridad nuestro adversario elegira
// otro movimiento mas arriba o, lo que es lo mismo, este
// movimiento no pertenece a la variante principal. Este
// fenomeno se conoce tambien como Fail-high. Por contra,
// Fail-low es cuando todas las jugadas nos dan un valor
// por debajo del Alfa inicial, lo que significa que esta
// posicion es peor de lo que esperabamos. Este fenomeno
// de Fail-low no puede producirse en este programa, pues
// search se llama siempre con un intervalo -10000,+10000
// valores que representan -mate,+mate

int search(int alpha, int beta, int depth)
{
	int i, j, x;
	BOOL c, f;
	
	/* Si hemos llegado a profundidad 0, llamamos a quiesce()
	   que nos devuelve un valor razonable de la posicion */
	if (!depth)
		return quiesce(alpha,beta);
	++nodes;
	pv_length[ply] = ply;

	/* Si estamos en jaque, nos interesa ampliar la profundidad
	   de busqueda */
	// A esto se le llama "Extensiones", en este caso de jaque
	// Tambien se pueden implementar extensiones de recaptura,
	// de avances de peones en finales, y muchas otras
	c = in_check(side);
	if (c)
		++depth;
	gen();
	if (follow_pv)  /* Estamos siguiendo la Variante Principal (PV)? */
		sort_pv();
	f = FALSE;

	/* iterar sobre los movimientos */
	for (i = gen_begin[ply]; i < gen_end[ply]; ++i) {
		sort(i);
		if (!makemove(gen_dat[i].m.b))
			continue;
		f = TRUE;
		x = -search(-beta, -alpha, depth - 1);
		takeback();
		if (x > alpha) {

			/* Este movimiento causo un cutoff, asi que incrementamos
			   el valor de historia para que sea ordenado antes la
			   proxima vez que se busque */
			history[(int)gen_dat[i].m.b.from][(int)gen_dat[i].m.b.to] += depth;
			if (x >= beta)
				return beta;
			alpha = x;

			/* actualizar la PV */
			// Se basa en una estructura de "array triangular" donde,
			// para cada ply, incluimos la jugada actual y despues,
			// el resto de las jugadas de PV del ply siguiente.
			// Observese que este array se actualiza a la vuelta de
			// las llamadas recursivas, es decir, cuando vamos "hacia
			// arriba" en el arbol
			pv[ply][ply] = gen_dat[i].m;
			for (j = ply + 1; j < pv_length[ply + 1]; ++j)
				pv[ply][j] = pv[ply + 1][j];
			pv_length[ply] = pv_length[ply + 1];
		}
	}

	/* Si no hay jugadas legales, estamos en jaque mate o
	   ahogados */
	if (!f) {
		if (c)
			return -10000 + ply;
		else
			return 0;
	}

	/* regla de las 50 jugadas */
	// Se compara con 100 y no con 50 porque estamos midiendo
	// "medias-jugadas", y 50 jugadas, son 100 medias-jugadas :)
	if (fifty >= 100)
		return 0;
	return alpha;
}


/* quiesce() es una busqueda recursiva minimax con alfa-beta
   cutoffs; en otras palabras, negamax. Basicamente solo
   extudia series de capturas y permite a la funcion de
   evaluacion cortar la busqueda (y establecer alfa). La idea
   es encontrar una posicion donde no haya mucho "en marcha"
   (es decir, no quedarnos en mitad de una secuencia de cambios
   de piezas), para que la evaluacion estatica funcione
   correctamente */
// Que la evaluacion pueda cortar la busqueda significa que
// no estamos obligados a elegir una captura. Si la evaluacion
// estatica es lo bastante buena, significa que hemos terminado
// los cambios de piezas en buena situacion, y podemos probablemente
// efectuar un movimiento tranquilo que no implique mas cambios.

int quiesce(int alpha,int beta)
{
	int i, j, x;
	BOOL c;  /* en jaque */
	BOOL f;  /* encontrada jugada legal */

	++nodes;
	pv_length[ply] = ply;
	c = in_check(side);

	/* si estamos en jaque, intentamos todas las jugadas
	   legales para escapar de el (no es necesario,
	   simplemente lo decidi asi */
	if (c)
		gen();

	/* En caso contrario, utilzamos la funcion de evaluavion */
	else {
		x = eval();
		if (x >= beta)
			return beta;
		if (x > alpha)
			alpha = x;
		gen_caps();
	}
	if (follow_pv)  /* Seguimos la PV? */
		sort_pv();
	f = FALSE;

	/* iterar sobre los movimientos */
	for (i = gen_begin[ply]; i < gen_end[ply]; ++i) {
		sort(i);
		if (!makemove(gen_dat[i].m.b))
			continue;
		f = TRUE;  /* Encontramos una jugada legal! */
		x = -quiesce(-beta, -alpha);
		takeback();
		if (x > alpha) {
			if (x >= beta)
				return beta;
			alpha = x;

			/* actualizar la PV */
			pv[ply][ply] = gen_dat[i].m;
			for (j = ply + 1; j < pv_length[ply + 1]; ++j)
				pv[ply][j] = pv[ply + 1][j];
			pv_length[ply] = pv_length[ply + 1];
		}
	}

	/* Si estamos en jaque, y no hay jugadas legales,
	   en fin, hemos perdido */
	if ((!f) && c)
		return -10000 + ply;
	return alpha;
}


/* sort_pv() se llama cuando la funcion de busqueda esta siguiendo
   la PV (Principal Variation). Recorre la lista de jugadas
   disponibles para ver si la jugada de la PV esta alli.
   En ese caso, aade 10,000,000 al valor de la jugada para
   que sea analizada en primer lugar. Si no follow_pv es FALSE
   y la funcion de busqueda deja de llamar a sort_pv() */

void sort_pv()
{
	int i;

	follow_pv = FALSE;
	for(i = gen_begin[ply]; i < gen_end[ply]; ++i)
		if (gen_dat[i].m.u == pv[0][ply].u) {
			follow_pv = TRUE;
			gen_dat[i].score += 10000000;
			return;
		}
}


/* sort() busca en la lista de jugadas desde 'from'
   hasta el final para tratar de encontrar el de mayor
   valor. Luego los intercambia para tener al principio
   el mejor movimiento, con la esperanza de producir un
   cutoff */

void sort(int from)
{
	int i;
	long bs;  /* mejor valor */
	int bi;  /* mejor i */
	gen_t g;

	bs = -1;
	bi = from;
	for (i = from; i < gen_end[ply]; ++i)
		if (gen_dat[i].score > bs) {
			bs = gen_dat[i].score;
			bi = i;
		}
	g = gen_dat[from];
	gen_dat[from] = gen_dat[bi];
	gen_dat[bi] = g;
}
